package sse

import (
	"bufio"
	"encoding/json"
	"fmt"
	"github.com/gofiber/fiber/v2"
	"go.uber.org/zap"
	"time"
)

type SSEService struct {
	dispatcher *ConnectionDispatcher
	producer   *SSEProducer
	consumer   *SSEConsumer
	logger     *zap.Logger
}

func NewSSEService(redisAddr, redisPassword string, redisDB int, bufferSize int, logger *zap.Logger) (*SSEService, error) {
	dispatcher := NewConnectionDispatcher(bufferSize)

	producer, err := NewSSEProducer(redisAddr, redisPassword, redisDB)
	if err != nil {
		return nil, fmt.Errorf("failed to create producer: %w", err)
	}

	consumer, err := NewSSEConsumer(redisAddr, redisPassword, redisDB, dispatcher)
	if err != nil {
		return nil, fmt.Errorf("failed to create consumer: %w", err)
	}

	service := &SSEService{
		dispatcher: dispatcher,
		producer:   producer,
		consumer:   consumer,
		logger:     logger,
	}

	return service, nil
}

func (s *SSEService) Start() {
	s.consumer.Start()
}

func (s *SSEService) Stop() {
	s.consumer.Stop()
	s.producer.Stop()
}

// для отправителей - допустим вотчер монги исал бы сюда
func (s *SSEService) SendEvent(userID, eventType string, data interface{}) error {
	return s.producer.SendEvent(userID, eventType, data)
}

// todo для тех кто потребляет контент
func (s *SSEService) HandleSSEConnection(ctx *fiber.Ctx, userID, deviceID string) error {
	ctx.Set(fiber.HeaderContentType, "text/event-stream")
	ctx.Set("Cache-Control", "no-cache")
	ctx.Set("Connection", "keep-alive")
	ctx.Set("Transfer-Encoding", "chunked")

	conn := s.dispatcher.AddConnection(userID, deviceID)
	mainCtx := ctx.Context().Done()

	ctx.Status(fiber.StatusOK).Context().SetBodyStreamWriter(func(w *bufio.Writer) {
		defer s.dispatcher.RemoveConnection(userID, deviceID)
		pingTicker := time.NewTicker(5 * time.Second)
		defer pingTicker.Stop()

		for {
			select {
			case <-mainCtx:
				s.logger.Info("Client disconnected", zap.String("userID", userID), zap.String("deviceID", deviceID))
				return
			case event, ok := <-conn.Channel:
				if !ok {
					s.logger.Info("Connection channel closed", zap.String("userID", userID), zap.String("deviceID", deviceID))
					return
				}

				payload, err := json.Marshal(map[string]interface{}{
					"event": event.EventType,
					"data":  event.Data,
				})
				if err != nil {
					s.logger.Error("Failed to marshal event", zap.Error(err))
					continue
				}

				if _, err := fmt.Fprintf(w, "data: %s\n\n", payload); err != nil {
					s.logger.Error("Failed to write event to stream", zap.Error(err))
					return
				}
				if err := w.Flush(); err != nil {
					s.logger.Error("Failed to flush SSE stream", zap.Error(err))
					return
				}
			case <-pingTicker.C:
				if _, err := fmt.Fprintf(w, "data: %s\n\n", `{"event":"ping"}`); err != nil {
					s.logger.Error("Failed to write ping to stream", zap.Error(err))
					return
				}
				if err := w.Flush(); err != nil {
					s.logger.Error("Failed to flush ping", zap.Error(err))
					return
				}
			}
		}
	})

	return nil
}
